/**************************************************************************/
/*                                                                        */
/*              IBM OS/2(tm) System Performance Monitor/2                 */
/*                            Version 2.0                                 */
/* (c) Copyright International Business Machines Corporation, 1992        */
/*                                                                        */
/**************************************************************************/
/*********** SPMAPI Common Header Include File ****************************/
/*                                                                        */
/*  This file includes:    Entry Point Definitions                        */
/*                         Manifest Constants                             */
/*                         Structure Definitions                          */
/*                         Logfile Format Definitions                     */
/*                                                                        */
/*  The entry points are:                                                 */
/*                         SPMAPIClose                                    */
/*                         SPMAPIGetData                                  */
/*                         SPMAPIInit                                     */
/*                         SPMAPIOpen                                     */
/*                         SPMAPIQuery                                    */
/*                         SPMAPISet                                      */
/*                         SPMAPITerm                                     */
/*                         SPMAPIQualifyFileName                          */
/*                         Datetime functions:                            */
/*                            datetime                                    */
/*                            datepart                                    */
/*                            timepart                                    */
/*                            str2dt                                      */
/*                            str2dtNonNLS                                */
/*                            dt2str                                      */
/*                            dv2str                                      */
/*                            tv2str                                      */
/*                                                                        */
/**************************************************************************/

/*
  Three Mode Include File:

    To compile with IBM C/2, or other compatible compilers, no special
    defines are required. Only the Large/Far model is supported.

    To compile with the C/SET-2 compiler, the #define INCL_32 is
    required to generate appropriate pragmas and declarations.

    To compile with the C386 compiler the #define C386 is required
    along with INCL_32. This compiler uses SDK20, not TOOLKT20.
*/

#ifdef INCL_32
  #ifdef C386
    #define SPMBOOL BOOL
    #define SPMPTR _far16
    #define SPMINT short
    #define SPMAPI_ENTRY _far16 _pascal
    #pragma pack(1)
  #else
    #define SPMBOOL BOOL16
    #define SPMPTR _Seg16
    #define SPMINT short
    #define SPMAPI_ENTRY _Far16 _Pascal
    #pragma pack(1)
  #endif
#else
  #define SPMBOOL BOOL
  #define SPMPTR
  #define SPMINT int
  #define SPMAPI_ENTRY EXPENTRY
#endif

/*

  SPMAPI - External API for SPM/2 V2.00

  Notes on the functionality of the API:

    1. Please use the manifest constants provided. While we do not
       expect that we would ever break code dependent upon our
       interface within a release, System Management is a quickly
       evolving area.

    2. SPM uses dates, times and datetime fields which are stored as
       floating point numbers. The numbers that are used are compatible
       with the time value numbers used by most statistical packages.

       The set of time conversion routines which are available
       include:

       typedefs:

         FPDATETIME - a double containing the number of
                      seconds and parts of seconds since 1Jan1960.

         FPDATE     - a float containing the number of days
                      since 1Jan1960.

         FPTIME     - an elapsed time containing the number of seconds
                      and parts of seconds since the start of the
                      time period. This is the normal SPM time element.
                      It is implemented as a float.

       Functions:

         datetime() returns the current FPDATETIME.

         datepart() returns an FPDATE (in a double) from an FPDATETIME.

         timepart() returns an FPTIME (in a double) from an FPDATETIME.

         dt2str() returns a date/time string from an FPDATETIME.

         dv2str() returns a date string from an FPDATE.

         tv2str() returns a date/time string from an FPTIME.

         str2dt() converts a country specific date/time string to an
                    FPDATETIME

         str2dtNonNLS() converts a non-country specific string to an
                    FPDATETIME

    3. The log files and records returned across the real-time
       interface are described in this file and SPMCTRGP.H.  The
       records are returned as a fixed format header and a set of
       additional segments which represent groups being collected.
       All system groups are predefined and delivered with SPM/2.
       Additional groups can be added by IBM, other vendors or the
       end user as User Hooks.  Adding a User Hook will generate a new
       SPMCTRGP.H file which will include all locally known
       definitions of collection Groups.


    4. Collection Groups appear only if they are active during a
       collection session.  There is no guarantee that
       any particular group will be in a particular record.

       In general a reporting program will look at the header of a
       record to determine if the record contains information about a
       Managed Session of interest (each record contains data from
       only one Managed Session) and if interested, will scan the
       substructures collected for that Managed Session for
       interesting data. If a Managed Session does not supply data in
       a reporting period, there may not be a record for it.  If a
       Managed Session supplies a large amount of data in a reporting
       period, there may be two or more records which contain data for
       the same Managed Session in the same reporting period.


    5. The Managing Session tends to collect and redistribute whatever
       data it receives. There is no guarantee of anything other than
       the guarantee that the structure of the data returned by the
       interface will be consistent with the definition.

       Lost information is not uncommon in collection systems of this
       type, and SPM/2 is no exception. There are a set of indicators
       in the collection record which show what, if any, problems were
       encountered with a Managed Session in the period.

*/

/*
  These constants are used within the API and may be of interest
  to the programmer.
*/
#define API_GENERAL_TIMEOUT     30000
#define ILOG_GENERAL_TIMEOUT    15000
#define ILOG_HANDSHAKE_TIMEOUT  30000
#define SPMAPI_PGM_NAME         "spmilog.exe"

/*
  Definitions for API command constants:
*/
#define SPMAPI_DRAIN                 1
#define SPMAPI_DRAIN_ALL             2
#define SPMAPI_HALT                  3
#define SPMAPI_HALT_ALL              4
#define SPMAPI_ALLOC                15

/*
  Definitions for API Set Constants:
*/
#define SPM_SET_RECORD_MULT          18

/*
  Definitions for API Query Constants:
*/
#define SPM_QU_DCF_LIST              5
#define SPM_QU_ILOG_LIST             6
#define SPM_QU_ACTIVE_DCF_NODES      7
#define SPM_QU_ILOG_STAT             8
#define SPM_QU_CFGD_GROUPS           9
#define SPM_QU_CFGD_DCF_NODES       10
#define SPM_QU_CFGD_PERIODS         11
#define SPM_QU_L_ILOG_LIST          16
#define SPM_QU_ILOG_NAME            17
#define SPM_QU_RECD_DCF_NODES       18
#define SPM_QU_HIST_GROUPS          19
#define SPM_QU_CFGD_TIMES           20
#define SPM_QU_CFGD_GROUPS_PERIOD      21
#define SPM_QU_CFGD_DCF_NODES_PERIOD   22
#define SPM_QU_CFGD_TIMES_PERIOD       23

/*
  Old Definitions for API Query Constants - for compatibility only:
*/
#define SPMAPI_QUDCFLIST             SPM_QU_DCF_LIST
#define SPMAPI_QUILOGLIST            SPM_QU_ILOG_LIST
#define SPMAPI_QUILOGNODES           SPM_QU_ACTIVE_DCF_NODES
#define SPMAPI_QUILOGSTAT            SPM_QU_ILOG_STAT
#define SPMAPI_QULOGGROUPS           SPM_QU_CFGD_GROUPS
#define SPMAPI_QULOGNODES            SPM_QU_CFGD_DCF_NODES
#define SPMAPI_QULOGPERIODS          SPM_QU_CFGD_PERIODS
/********************************************************************/

/* Notes on Returned Informaiton:

     All API functions provide extended return codes when an error
     occurs:

     1. The actual return of the function is SPMBOOL indicating overall
        failure if TRUE or success if FALSE.

     2. If TRUE is returned, additional information may be returned:

        a. A function code indicating what was being done when the
           error occurred. This is returned in the area provided by
           the caller for the extended return code.

        b. A function return code which indicates the actual error
           which was encountered. This is normally an error from an
           underlying function.

     3. If FALSE is returned, the extended return values are
        reserved.

*/

typedef struct _EXTENDEDRC
{
  USHORT FunctionCode;  /* the function being executed when... */
  USHORT FunctionRC;    /* this error resulted */
} EXTENDEDRC;

/*
  Definitions for Function Codes returned by the API:
*/

#define SPMAPI_PERR_BUFFERAREA       1
#define SPMAPI_PERR_BUFFERLENGTH     2
#define SPMAPI_PERR_BYTESREAD        3
#define SPMAPI_PERR_CONFIGDATA       4
#define SPMAPI_PERR_CONFIGOPT        5
#define SPMAPI_PERR_DOSEXECPGM       6
#define SPMAPI_PERR_FILTER           7
#define SPMAPI_PERR_FILTERSPEC       8
#define SPMAPI_PERR_LOGFILE          9
#define SPMAPI_PERR_LOGHANDLE       10
#define SPMAPI_PERR_LOGOBJHANDLEI   11
#define SPMAPI_PERR_LOGOBJHANDLEO   12
#define SPMAPI_PERR_LOGOBJNAME      13
#define SPMAPI_PERR_QUCOMPNAME      14
#define SPMAPI_PERR_QUCOMPONENT     15
#define SPMAPI_PERR_QUOPTION        16
#define SPMAPI_PERR_QURESULTS       17
#define SPMAPI_PERR_RETURNCODE      18
#define SPMAPI_PERR_SHUTDOWN        19
#define SPMAPI_PERR_SHUTDOWNOPT     19
#define SPMAPI_PERR_LOGOBJECT       20
#define SPMAPI_PERR_USERPARM        21
#define SPMAPI_PERR_FILENAME        22

/*
  The following codes may be returned by a query in the
  function code field when an operation is otherwise successful
*/
#define SPMAPI_INFO_FILEREPAIRED    40
#define SPMAPI_INFO_FILEINUSE       41


/*
  if the function code returned is DOSCALL, then the RC returned
  is the failure code from the underlying OS/2 call.

  if the function code returned is NBQCALL, then the RC returned
  is the failure code from the underlying NETBIOS call.

  if the function code returned is ILGPIPE, then the RC returned is
  the failure code from the Managing Session IPC mechanism. These
  codes indicate an internal error and should be handled as such. No
  recovery is available at this time.
*/

#define SPMAPI_PERR_ILGPIPE        253
#define SPMAPI_PERR_NBQCALL        254
#define SPMAPI_PERR_DOSCALL        255

/*
  Definitions for Return Codes for the API:
*/
#define SPMAPI_ERR                   1
#define SPMAPI_ERROR                 1
#define SPMAPI_ERR_DOSALLOCSHRSEG    2
#define SPMAPI_ERR_DOSCLOSE          3
#define SPMAPI_ERR_DOSCLOSESEM       4
#define SPMAPI_ERR_DOSCREATESEM      5
#define SPMAPI_ERR_DOSEXECPGM        6
#define SPMAPI_ERR_DOSGETSHRSEG      7
#define SPMAPI_ERR_DOSOPENSEM        8
#define SPMAPI_ERR_DOSSEMREQUEST     9
#define SPMAPI_ERR_FILEBADFORMAT    10
#define SPMAPI_ERR_FILEINUSE        11
#define SPMAPI_ERR_FILENOTINUSE     12
#define SPMAPI_ERR_GDBUFFERAREA     13
#define SPMAPI_ERR_GDBUFFERLENGTH   14
#define SPMAPI_ERR_GDBYTESREAD      15
#define SPMAPI_ERR_GDRETURNCODE     16
#define SPMAPI_ERR_GDSEMAPHORE      17
#define SPMAPI_ERR_GETGDA           18
#define SPMAPI_ERR_GETIDA           19
#define SPMAPI_ERR_INTERFACE_BUSY   20
#define SPMAPI_ERR_NAKTIMEOUT       21
#define SPMAPI_ERR_NOSPACE          22
#define SPMAPI_ERR_NOT_FOUND        23
#define SPMAPI_ERR_NOT_OPEN         24
#define SPMAPI_ERR_NOT_SUPPORTED    25
#define SPMAPI_ERR_NO_SPACE         26
#define SPMAPI_ERR_QUOPTIONNOTVALID 27
#define SPMAPI_ERR_RANGE            28
#define SPMAPI_ERR_TIMEOUT          29
#define SPMAPI_ERR_TOOMANY          30
#define SPMAPI_ILG_FOUND            31
#define SPMAPI_ILG_NOTFOUND         32
#define SPMAPI_NDE_FOUND            33
#define SPMAPI_NDE_NOTFOUND         34
#define SPMAPI_NOT_ALLOC            35
#define SPMAPI_ERR_OBJECTINUSE      36
#define SPMAPI_ERR_OBJECTNOTINUSE   37
#define SPMAPI_ERR_NULL_PARM        38
#define SPMAPI_ERR_NOT_TERMINATED   39

/*
  General Functions and Structures used with SPM/2 API Calls:
*/


#ifndef FPDATETIME_DEFINED
  /* FPDATETIME:
    this time is the number of seconds and fractions of a second
    since 1Jan1960.
  */
  typedef double FPDATETIME; /* dt */

  /* DATE:
    this item is the number of days since 1Jan1960.  To convert to
    FPDATETIME multiply by 24*60*60.
  */
  typedef float FPDATE; /* date */

  /* TIME:
    this time is the number of seconds and parts of seconds.
  */
  typedef float FPTIME; /* time */

  #define FPDATETIME_DEFINED
#endif

#ifndef SPM_DATETIME_DEFINED
  /* TIMEPERIOD:
    this time is a number of seconds and parts of a second.
  */
  typedef FPTIME TIMEPERIOD; /* tp */

  /* TOD:
    this time is a number of seconds and fractions of a second
    since midnight.
  */
  typedef FPTIME TOD; /* tod */
  #define SPM_DATETIME_DEFINED
#endif

#ifndef DATETIME_FUNCTIONS_DEFINED
  #define DTERROR -1.7e+308

  /*
    datetime provides a reasonably rich conversion set, but it is
    limited to the length of USA English month names and SBCS date and
    time delimiters.  If abbreviations of months are not 3 characters
    or if a month name is more than 9 characters or if a day of the
    week is longer than 9 problems may be encountered.

    To provide National Language Support / Double Byte Character Set
    compatibility to programs which are NLS/DBCS aware, there are
    non-country specific functions which allow the program to control
    all formatting.  Look for the phrase Non-NLS to find these
    functions.

    The only restriction with the Non-NLS routines is that the local
    country uses Arabic numbers, the Gregorian Calendar and UTC style
    times.
  */

  double SPMAPI_ENTRY datetime(void);
  /* returns the current datetime value */

  double SPMAPI_ENTRY datepart(double);
  /* returns a date value from a datetime value*/

  double SPMAPI_ENTRY timepart(double);
  /* returns the time of a datetime variable */

  double SPMAPI_ENTRY str2dt(char * SPMPTR);
  /* converts the string to a datetime variable, returns DTERROR if invalid */

  /*
    The format of a date string is country specific. Slash (/) and colon (:)
    are country dependent and are ignored.  Brackets denote optional fields.
    Some examples are:

      USA:              MM/DD/YYYY[:HH:MM:SS]

      Turkey:           DD/MM/YYYY[:HH:MM:SS]

      Japan:            YYYY-MM-DD[:HH:MM:SS]

  */

  double SPMAPI_ENTRY str2dtNonNLS(char * SPMPTR);
  /* converts the string to a datetime variable, returns DTERROR if invalid */

  /*
    The format of the date string is not country specific.  It must
    contain no separators and be in the order described below. Brackets
    denote optional fields.

      The format must be YYYYMMDD[HHMMSSTTTT].

  */

  PSZ SPMPTR SPMAPI_ENTRY dt2str(char * SPMPTR,double,SPMINT);
  /* convert datetime to printable string.

       The order of the date components and the separators is
       determined by the existing country setup. USA English is
       shown here.  Others will be similar.

       length is the key to the format.
      -18          - Non-NLS string: YYYYMMDDHHMMSSTTTT (18+1 bytes)
       less than 6 - *'s to show error
       6           - mmddyy
       7           - ddmmmyy
       8 or 9      - mm/dd/yy
       10          - ddmmmyy:hh
       11          - mm/dd/yyyy
       12          - mm/dd/yy:hh
       13          - ddmmmyy:hh:mm
       14 or 15    - mm/dd/yy:hh:mm
       16          - ddmmmyy:hh:mm:ss
       17 to 18    - mm/dd/yy:hh:mm:ss
       19 to 22    - mm/dd/yyyy:hh:mm:ss
    The following are only guaranteed to work for USA English
       23          - September dd, yyyy
       24 to 26    - September dd, yyyy hh:mm
       27 or more  - September dd, yyyy hh:mm:ss
  */

  PSZ SPMPTR SPMAPI_ENTRY dv2str(char * SPMPTR,double,SPMINT);
  /* convert dateval to printable string.
       length is the key to how it is formatted.
      -8           - Non-NLS string: YYYYMMDD (8+1 bytes)
       less than 6 - *'s to show error
       6           - mmddyy
       7           - ddmmmyy
       8           - mm/dd/yy
       9           - ddmmmyyyy
       10          - mm/dd/yyyy
       11          - yyyy/mm/dd
    The following are only guaranteed to work for USA English
       12          - mmm dd, yyyy
       13          - Mon mm/dd/yy
       14          - Mon yy/mm/dd
       15          - Mon mm/dd/yyyy
       16          - Mon yyyy/mm/dd
       17          - Mon mmm dd, yyyy
       18          - Mon, mmm dd, yyyy
       19          - Mon: mmm dd, yyyy
       20          - September dd, yyyy
       21          - September dd, yyyy
       22          - Mon September dd, yyyy
       23          - Mon, September dd, yyyy
       24          - Mon: September dd, yyyy
       25          - Mon: September dd, yyyy
       26          - Mon: September dd, yyyy
       27          - Mon: September dd, yyyy
       28          - Wednesday September dd, yyyy
       29          - Wednesday, September dd, yyyy
       30          - Wednesday: September dd, yyyy
  */

  PSZ SPMPTR SPMAPI_ENTRY tv2str(char * SPMPTR,double,SPMINT);
  /* convert timeval to printable string.
       length is the key to how it is formatted.
      -12          - Non-NLS string: HHHHMMSSTTTT (12+1 bytes)
       less than 2 - *'s to show error
       2           - hh
       3           - hhh
       4           - hhmm
       5           - hh:mm
       6           - hhh:mm
       7           - hhhh:mm
       8           - hh:mm:ss
       9           - hhh:mm:ss
       10          - hh:mm:ss.t
       11          - hh:mm:ss.tt
       12          - hh:mm:ss.ttt
       13          - hh:mm:ss.tttt
       14          - hhh:mm:ss.tttt
       15          - hhhh:mm:ss.tttt
  */
  #define DATETIME_FUNCTIONS_DEFINED
#endif

typedef EXTENDEDRC * SPMPTR PEXTENDEDRC;

typedef struct _FILTERSPEC
/*
  Determines the range in the log that will be read or indicates that
  real-time data is to be returned.

  It is also the underlying structure in the vector that is returned
  on a Query request that returns the logging intervals on a specific
  log file.
*/
{
  FPDATETIME dtStartDateTime;
  FPDATETIME dtEndDateTime;
}FILTERSPEC;

typedef FILTERSPEC * SPMPTR PFILTERSPEC;

typedef PFILTERSPEC * SPMPTR PPFILTERSPEC;

typedef struct _LOGFILEHANDLE
{
  BYTE    cReserved[64];  /* handle of logfile */
} LOGFILEHANDLE;

typedef LOGFILEHANDLE * SPMPTR PLOGFILEHANDLE;

typedef USHORT QUOPTION;

typedef QUOPTION * SPMPTR PQUOPTION;

typedef struct _QUTIMES
{
  FPDATETIME dtStartTime;
  FPDATETIME dtEndTime;
}QUTIMES;

typedef QUTIMES * SPMPTR PQUTIMES;

typedef struct _QUERYHDR
{
  USHORT usResultMax;      /* filled in by the caller */
  USHORT usItemsReturned;  /* which may be zero if none are found */
  PVOID SPMPTR pbResult;   /* a pointer to the returned value(s) */
}QUERYHDR;

typedef QUERYHDR * SPMPTR PQUERYHDR;

typedef struct _SESSIONSPEC
{
  FILTERSPEC fsFilter;
  CHAR       szComponentName[1];
}SESSIONSPEC;

typedef SESSIONSPEC * SPMPTR PSESSIONSPEC;

typedef PSESSIONSPEC * SPMPTR PPSESSIONSPEC;

/********************************************************************/

/* SPMAPISet */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPISet(QUOPTION quoption,      /* in     */
                          PSZ SPMPTR pszComponentName, /* in     */
                          PVOID SPMPTR pbSetting,      /* in/out */
                          PEXTENDEDRC pxrcRC);         /* out    */




/*
  The set function supports changing the operation of components that
  are currently running on the local machine or on the network.  It does
  not require the caller to have issued an init or open, but that action
  might be required to be done by someone. Once a component name is
  known, its options can be changed. In general the prior
  setting is returned in the setting field, so the use of constants is
  of great danger.

  The user must provide a setting area that is correct for the request.
  There is no standard header on this area.

*/

/*
  SPM_SET_RECORD_MULT
  pbSetting is a pointer to a long which contains the new multiplier.
  The prior setting of the multiplier is returned in the same field.
  Setting a Managing Session to a zero multiplier turns off
  recording.

  szComponentName is required to be an active Managing Session name,
  including the suffix (see below for a discussion of suffixes).
*/


/********************************************************************/

/* SPMAPIQuery */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPIQuery(QUOPTION quOption,      /* in       */
                            PSZ SPMPTR pszComponentName, /* in - opt */
                            PVOID SPMPTR pbResults,      /* out      */
                            PEXTENDEDRC pxrcRC);         /* out      */



/*
  The query function supports finding out about what SPM
  components are currently running on the local machine or on the
  network. It also supports increasing levels of detailed queries,
  given that a component name is known.  It does not require the
  caller to have issued an init or open. Once a component name is
  known, more information about the component may be available.
  In the case of an inquiry about an individual component the
  component name must be supplied.

  For those queries which require a SESSIONSPEC instead of
  a component name, the SESSIONSPEC pointer may be cast to
  a PSZ to avoid compiler errors or warnings.   The
  affected queries are SPM_QU_CFGD_TIMES_PERIOD,
  SPM_QU_CFGD_DCF_NODES_PERIOD, and
  SPM_QU_CFGD_GROUPS_PERIOD.

  The user must provide a return area large enough for the
  result. The return area must begin with a QUERYHDR structure
  with the maximum size of the return area provided in the
  usResultMax field of this structure.

  If the usResultMax field is zero, the required result size is
  returned in this field (replacing the zero) along with the
  SPMAPI_ERR_NO_SPACE extended error. This field (usResultMax) is
  not changed if it was not zero on the call. If the usResultMax
  field was zero on the call, no other field in the structure is
  referenced. This is similar to the LAN Server/Requester API, but
  with fewer parm entries. The overall intention and operation is
  very similar.

  There is one caution in the use of the feedback of the length as
  described above: the system is dynamic. The length needed on the
  first call (to get the length) may not be correct for the next call
  to the function (the longer the interval between the calls, the more
  chance that things have changed).

  The actual format of the result that pbResult points at is
  dependent upon the request. The result is built in the area
  provided by the caller, though not necessarily right after the
  header. As much data as will fit is returned, if any. An
  attempt to terminate a truncated block is made, but is not
  guaranteed.  The number returned in usItemsReturned is the
  number that are in the return area.  The actual number
  available is equal to the number returned if the length of the
  return area is large enough to hold them all.
*/

/*
  pbResult can be cast to the type of return which is expected.
  If usItemsReturned is 0, pbResult is undefined.

  Many queries result in a list of items which are returned as a
  vector of the return type. If the fields are SZ they are
  terminated by an additional null field (making them SZZ).  In
  other cases a single value or structure might be returned. All
  of the available queries along with the structures that they
  return follow:
*/

/*
  Notes on Query:

    1. The name returned by a query to the Managing Session component
       returns the disk, path and filename by which the Managing
       Session knows the file.  If the Managing Session is on the
       local machine and the path specified at startup is complete, no
       problem results. If only a partial path was specified the
       startup process attempts to completely define the name. If the
       log is on a remote machine, the startup process attempts to
       resolve a UNC name for the log.  The probability that the name
       resolved is correct is extremely high, but it is suggested that
       complete path names be specified on non network machines and
       that UNC names be used in network situations.

    2. A query for the running Managing and Managed Sessions attempts
       to return all of the known names. There are situations in which
       all of names may not be returned on the first query, since the
       priority of the request and the service level of the request
       are both quite low.

       The location of the names is not limited to the current domain,
       nor is any check done to ensure that the requester is even
       running on the local machine. In most cases, this is not a
       problem, but if the requester is not running or the responding
       session is not in the current domain or if the responding
       session terminates after a query about it, the application must
       be capable of handling the situation.

       SPM/2 internally uses the Lan Server Computer Name or the
       constant "LOCAL" to identify a machine which is part of the
       collection process.  "LOCAL" is used when the machine name is not
       available from Lan Server.  Since there can be more than
       one SPM component associated with a particular machine, SPM/2
       appends a suffix to the name of the machine.

       Suffixes identify the type of SPM component. There are currently
       3 components defined:

         SPM/2 Agent (NBL)         - has a numeric suffix (0-9)
         External Query            - has a suffix of # (pound sign /
                                        number sign / 0x23)
         Managing Session (ILOG)   - has all other suffixes

       The name (with the suffix) can be up to 16 bytes long.

       In the simple case of local collection (from a cold start)
       would be:

         SPM/2 Agent:      LOCAL0
         Managing Session: LOCALA

    3. Query does not provide an inactive log name search. This is
       available via normal file directory search functions. A query
       against an inactive log verifies that the file is a valid
       log.

    4. Query actually reads the log file for both active and inactive
       requests that specify a file name. The current user must have
       at least read only permission granted to a log file that is
       to be queried or opened for history retrieval.
*/

/*
  SPM_QU_DCF_LIST
  pbResult is a pointer to a vector of null terminated names of the
  Agent nodes currently active on the network. The names include
  suffixes (e.g. xxxxxxxx0).
*/

/*
  SPM_QU_ILOG_LIST
  pbResult is a pointer to a vector of null terminated names of the
  Managing Sessions currently active on the network. The names include
  suffixes (e.g. xxxxxxxxA).
*/

/*
  SPMAPI_QU_L_ILOG_LIST
  pbResult is a pointer to a vector of null terminated names of the
  Managing Sessions currently active in the local machine. The names
  include suffixes (e.g. xxxxxxxxA).
*/

/*
  SPM_QU_ILOG_STAT
  pbResult is a pointer to a structure which contains the status of
  the Managing Session component named in the szComponentName field.

  szComponentName is required to be an active Managing Session name.
  The name must include the correct suffix (e.g. xxxxxxxxA).
*/
typedef struct _QUILOGSTAT
{
  FPDATETIME dtStarted;    /* the date and time the ilog started */
  FPTIME tvSample;         /* the sample interval */
  FPTIME tvRecording;      /* the recording interval */
  CHAR   szLogFileName[1]; /* the full path and file name of the log */
                           /* NOTE: The name returned is the name by
                              which the Managing Session knows the
                              file. If the file was opened with an UNC
                              name, it may be used directly, otherwise
                              mapping of the drive letter to the UNC
                              prefix or a locally aliased drive letter
                              is required.
                           */
}QUILOGSTAT;

typedef QUILOGSTAT * SPMPTR PQUILOGSTAT;

/*
  SPM_QU_CFGD_TIMES
  pbResult is a pointer to a structure which contains the configured
  times in the LogFile named in the szComponentName field.

  szComponentName is required to be a valid szLogFileName.
*/
typedef struct _QUCFGDTIMES
{
  FPTIME tvSample;    /* the sample interval */
  FPTIME tvRecording; /* the recording interval */
  FPTIME tvWSIntrvl;  /* the Working Set Interval */
}QUCFGDTIMES;

typedef QUCFGDTIMES * SPMPTR PQUCFGDTIMES;

/*
  SPM_QU_CFGD_TIMES_PERIOD

  Retrieves the times (collection frequency, logging
  frequency, and working set interval) for a given
  start/stop session of the log file.
  Use SPM_QU_CFG_PERIODS to retrieve the start/stop
  times of all sessions in the log file.

  pszComponentName points to a SESSIONSPEC structure,
  which defines the start/stop session.
  pbResult returns the times in the same format as
  SPM_QU_CFGD_TIMES above.
*/

/*
  SPMAPI_QU_ILOG_NAME
  pbResult is a pointer to a null terminated string which contains
  the name of the SPM Managing Session that is using the Log File.
  The name returned includes a suffix (e.g. xxxxxxxxA).

  szComponentName is required to be an active szLogFileName.
*/

/*
  SPM_QU_ACTIVE_DCF_NODES
  pbResult is a pointer to a vector of null terminated names of
  the machine names CURRENTLY being collected by the Managing Session.
  The names returned do not include a suffix.

  szComponentName is required to be an active Managing Session name.
  This name must include the suffix (e.g. xxxxxxxxA).
*/

/*
  SPM_QU_CFGD_DCF_NODES
  pbResult is a pointer to a vector of null terminated names of the
  machine names for which the Log is configured.  The names returned do
  not include a suffix.

  szComponentName is required to be a valid szLogFileName.
*/

/*
  SPM_QU_CFGD_DCF_NODES_PERIOD

  Retrieves the collected nodes for a given start/stop
  session of the log file.
  Use SPM_QU_CFG_PERIODS to retrieve the start/stop
  times of all sessions in the log file.

  pszComponentName points to a SESSIONSPEC structure,
  which defines the start/stop session.
  pbResult returns the nodes in the same format as
  SPM_QU_CFGD_DCF_NODES above.
*/

/*
  SPM_QU_RECD_DCF_NODES
  pbResult is a pointer to a vector of null terminated names of the
  machine names for which contact was established in any session in the
  log.  The names returned do not include a suffix.

  szComponentName is required to be a valid szLogFileName.
*/

/*
  SPM_QU_CFGD_GROUPS
  pbResult is a pointer to a vector of USHORT values which contain the
  ordinals of the collection groups for which the log is currently
  configured.

  szComponentName is required to be a valid szLogFileName.
*/

/*
  SPM_QU_CFGD_GROUPS_PERIOD

  Retrieves the collected groups (resources) for a given
  start/stop session of the log file.
  Use SPM_QU_CFG_PERIODS to retrieve the start/stop
  times of all sessions in the log file.

  pszComponentName points to a SESSIONSPEC structure,
  which defines the start/stop session.
  pbResult returns the nodes in the same format as
  SPM_QU_CFGD_GROUPS above.
*/

/*
  SPM_QU_HIST_GROUPS
  pbResult is a pointer to a vector of USHORT values which contain the
  ordinals of the collection groups for which the log has ever been
  configured. There is no guarantee that any data was collected for
  any of these ordinals.

  szComponentName is required to be a valid szLogFileName.
*/

/*
  SPM_QU_CFGD_PERIODS
  pbResult is a pointer to a vector of type FILTERSPEC which
  contain the start and stop times of collection sessions on the
  log. If the log is active, the end time of the last member of the
  vector is 0E0.

  szComponentName is required to be a valid szLogFileName.
*/




/********************************************************************/

/* SPMAPIInit */

/********************************************************************/


SPMBOOL SPMAPI_ENTRY SPMAPIInit(PSZ SPMPTR pszLogFileName, /* in */
                             LONG        lRMultOverRide,  /* in  */
                             PEXTENDEDRC pxrcRC);         /* out */

/*
  SPMAPIInit starts a Managing Session instance. Since only one log is
  processed by any given Managing Session, and the information
  required to start the instance is contained in the header of the
  log, the log must not already be active. The instance is started on
  the local machine.

  pszLogFileName is required and must indicate an inactive but
  configured Log File.

  lRMultOverRide is normally set to -1 to indicate that the default
  recording multiplier is to be used. It can also be set to 0 to
  cause recording of interval samples to be skipped. Other values
  can be used to change the recording multiplier at startup.
*/


/********************************************************************/

/* SPMAPIOpen */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPIOpen(PSZ SPMPTR   pszLogFileName,  /* in  */
                           PFILTERSPEC       pfsFilters,      /* in  */
                           PLOGFILEHANDLE    pLogHandle,      /* out */
                           PEXTENDEDRC       pxrcRC);         /* out */


/*
  SPMAPIOpen enables the input processing of an active or
  inactive log.  The pszLogFileName must be a valid log file
  name.  pfsFilters must point to a valid FILTERSPEC struct.  If
  no data is available in the time frame selected, EOF is
  indicated on the first GetData after the open.

  pszLogFileName is required and must indicate a configured Log
  File which can be either active or inactive.

  pfsFilters points to a valid FILTERSPEC structure. It contains
  the start and stop time of the interval to be read.

  As a special case, if the DT values for both the start and stop
  times of the FILTERSPEC structure are both 0e0 and the Managing
  Session which is associated with the LogFileName is active, realtime
  data in the same format as historical data will be returned.

  To read all of the data on a log, the start DT should be set
  to 0e0 and the end DT should be set to the current DT
  via the datetime() function.  It is an error to set the end
  DT to 0e0 if the start DT is not 0e0.

  pLogHandle is a pointer to a log handle structure provided by
  the user.  A pointer to this log handle structure is required
  in subsequent GetData, PutData and Close requests and is the
  analog of a file handle. Don't rely on the fields in the
  structure.

*/


/********************************************************************/

/* SPMAPIGetData */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPIGetData(PLOGFILEHANDLE pLogHandle,  /* in       */
                              PVOID SPMPTR     pbLogRec,       /* out      */
                              USHORT           usLogRecMaxLen, /* in       */
                              PUSHORT SPMPTR   pusLogRecLen,   /* out      */
                              PVOID            pReserved1,     /* reserved */
                              PVOID            pReserved2,     /* reserved */
                              PEXTENDEDRC      pxrcRC);        /* out      */



/*
  SPMAPIGetData returns the next instance of sample data from a
  managing session. An SPMAPIOpen must have been successfully executed
  to establish an open Managing Session handle.

  The SPMAPIOpen call specifies the range of data to be returned in
  the FILTERSPEC struct. If no data is available in the time frame
  selected, EOF is indicated on the first GetData after the open by
  setting usLogRecLen to zero.

  pLogHandle is a pointer to a log handle structure provided by the
  user and filled in by the SPMAPIOpen call.  The results of modifying
  this field are undefined.

  pbLogRec is a byte aligned area into which the next record is read.
  Its maximum length is indicated by usLogRecMaxLen.

  pusLogRecLen is set to the actual number of bytes read. EOF is
  indicated by a zero in this field. EOF is not considered an error.
  EOF is indicated when the next record exceeds the FILTERSPEC or (in
  realtime mode) the Managing Session terminates or otherwise cannot
  be contacted.

  Two fields, pReserved1 and pReserved2 are reserved for future
  function. They should currently be (PVOID)NULL pointers.

  Notes on SPMAPIGetData:

    1. If the SPMAPIOpen indicated that real-time data was to be
       returned, the call to SPMAPIGetData blocks until that data is
       available or an error is returned by the Managing Session.
       Since the time involved could be long, it is suggested that the
       call to SPMAPIGetData be issued in a separate thread from the
       one that processes PM messages. To cleanly terminate an
       outstanding read, use SPMAPIClose to force completion of the
       read and ensure that the thread issuing the read terminates
       when it receives an End Of File (EOF) indication. EOF is
       defined as zero bytes returned from an SPMAPIGetData.  Failure
       to detect an EOF condition (which is not an error) may result
       in a loop through this interface.

    2. In real-time mode, all data received by the Managing Session is
       returned in the order which it was received. Network delays may
       make this inconsistent between managed machines. Data may be
       lost if the managing session connection is not serviced in a
       timely manner.

    3. If insufficient space is available in the area provided by the
       caller of SPMAPIGetData, slightly different actions are taken
       by the API depending upon the mode of retrieval.

       In the case of real-time data, the data is lost. In the case of
       historical data, the API will attempt to deliver the data which
       caused the error on the next SPMAPIGetData. In both cases, the
       data returned (if any) by the call which gets the error is
       undefined. There is a manifest constant, SPM_TYPE10_MAX_SIZE,
       which contains the maximum size of any record which will be
       returned by this interface.

    4. Underlying errors from the file system are returned if
       encountered while processing historical data. These errors are
       the same as those returned from DosRead. The method used to
       return real-time data is not defined and therefore any
       underlying errors are reflected as an EOF.

    5. Once an error or EOF is returned by the interface, EOF is
       returned on all subsequent calls. To recover or reposition, the
       Managing Session handle should be closed and reopened.

*/


/********************************************************************/

/* SPMAPIClose */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPIClose(PLOGFILEHANDLE pLogHandle, /* in */
                            PEXTENDEDRC    pxrcRC);        /* out */


/*
  SPMAPIClose terminates the connection to a log file. If the log file
  was being read on a historical basis, it is closed. If the attachment
  was realtime, the attachment is terminated, but the managing session
  is not affected. If the logfilehandle is invalid, an error is
  returned.
*/


/********************************************************************/

/* SPMAPITerm */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPITerm(PSZ SPMPTR pszLogFileName, /* in */
                           USHORT      usOptions,         /* in  */
                           PEXTENDEDRC pxrcRC);           /* out */


/*
  SPMAPITerm terminates one or more Managing Sessions. The LogFileName
  is required for a DRAIN or HALT operation. An ALL operation affects
  all Managing Sessions on the local machine (if any) and does not
  require a valid LogFileName.

  In general, DRAIN allows the current operations to continue to
  completion. A HALT terminates all operations without cleanup.
*/



/********************************************************************/

/* SPMAPIQualifyFileName */

/********************************************************************/

SPMBOOL SPMAPI_ENTRY SPMAPIQualifyFileName
                 (PSZ SPMPTR pszFileName, /* input - file name to be
                                             qualified */
                  USHORT usQNameLen,      /* input - size of output area */
                  PSZ SPMPTR pszQName,    /* output - fully qualified name */
                  PEXTENDEDRC pxrcRC);    /* output - return codes */


/*
  QualifyFileName takes as input a file name with any level of
  qualification and returns a fully qualified file name. The search
  order is the current directory then the DPATH. If the drive specified
  on the input file name is a network drive, QualifyFileName returns
  the UNC name of the file (if it can be found). If the drive is a
  local drive that is known on the network as a different drive letter
  and the local drive letter is specified, the return value is a local
  file name. UNC names are only returned for network drive letters
  currently in use on the machine on which the call is issued.
*/


/********************************************************************/

/* SPM Log File Description - User Level

/********************************************************************/

#ifndef SPM_NODENAME_SIZE
  #define SPM_NODENAME_LEN   15
  #define SPM_NODENAME_SIZE  (SPM_NODENAME_LEN+1)
  #define SPM_PWD_LEN        9
  #define SPM_PWD_SIZE       (SPM_PWD_LEN+1)
#endif

#ifndef SPM_PIDTID_DEFINED
  #define PID1632         ULONG
  #define TID1632         ULONG
  #define SPM_PIDTID_DEFINED
#endif

#ifndef SPM_BASICHDR_DEFINED
  /* BASICHDR:

      In general the log is a typical file but it contains a variety
      of record types. For RAS purposes, the log is readable as a
      sequential file of variable length records, each with a BASICHDR
      of the following form:
  */

  typedef struct _BASICHDR
  {
    USHORT usLL;               /* Non-Inclusive Length of this record */
                               /* Note: Non-Inclusive Length is
                                  used here because records are
                                  actually read with two DosRead()
                                  functions. The first read is
                                  always a BASICHDR and the next
                                  read is for the value contained
                                  in the usLL. If the RecordType or
                                  SubType is not interesting, the
                                  file can be positioned using the
                                  usLL directly.
                                */
    UCHAR  ucRecordSeries;      /*
                                  LLS: overall log format indicator.
                                  If the log file format changes in a
                                  way that precludes processing by a
                                  program written for a prior format,
                                  this field will indicate that fact.
                                  It is inadvisable to attempt to
                                  process a log whose series is not
                                  known to you.  The initial value is
                                  zero. This field is reserved.
                                */
    UCHAR  ucRecordType;        /* we do not expect a lot of record types */
    UCHAR  ucRecordSubType;     /* nor a lot of subtypes */
    /*
      Note:
        The record type and sub-type fields must be checked to ensure
        that they are the type and sub-type expected.  It is inadvisable
        to attempt to process a type or sub-type that is reserved.
        The API GetData call returns only the records documented here.
        All other record types and subtypes are reserved.
    */
  }BASICHDR;
  #define SPM_BASICHDR_DEFINED
#endif

#ifndef SPM_RBA_DEFINED
  /* RBA:
    an RBA is the Relative Byte Address of a record in the log file
  */
  typedef ULONG RBA; /* rba */
  #define SPM_RBA_DEFINED
#endif

#ifndef SPM_ROF_DEFINED
  /* ROF:
    a ROF is the byte offset of a field in the same record of the
    log file. It contains the number of bytes the field is from the
    first byte of the ROF field. The referenced field is always at
    a higher RBA than the ROF.

    Typical usage:

    pszTarget = (PSZ) ((PVOID)&rofSource + rofSource);

    Note: if rofSource = NULL then pszTarget automatically points
    to a 0x0000 field.

  */
  typedef USHORT ROF; /* rof, nrof */
  #define SPM_ROF_DEFINED
#endif

#ifndef SPM_VRM

  /* VRM:

       There are times when things change. To this end, SPM/2 provides
       indications as to the processability of a log file. Primary to
       this feature is the ucRecordSeries field in the basichdr.
       Version Release Modification status is intended to indicate less
       drastic changes and to alert a processing program to the
       potential impact of those changes.

       A change in Modification indicates that changes to the underlying
       formats of type 10 records were made that includes the addition
       or obsolescence of fields within one or more groups. Fields are
       always added at the end of a description. Obsolescent fields are
       set to zero.  Your current version will run if it provides for
       protection against errors caused by zeros where obsolescent
       fields are no longer being collected. Field offsets within
       records have not changed. Ordinals have not changed.

       A change in Release indicates that changes to the underlying
       formats of type 10 records were made that includes the addition
       or deletion of entire groups and the addition or obsolescence of
       fields within one or more groups. A new version is recommended.
       Changes within a group have the same limitations as a change in
       Modification level. Ordinals have been added or deleted, but
       prior ordinals within the same version have not changed.

       A change in Version indicates that changes to the underlying
       formats of type 10 records were made in a way that will
       probably render programs using prior versions of the log
       descriptions invalid. A new version is required. Ordinals
       have probably changed.

       VRM information is provided to the Managed Session from the
       Service Functions DLL.

       VRM information is reserved in SPM/2 2.0
  */
  typedef struct _VRM
  {
    UCHAR   ucVer;              /* the Version level of the record      */
    UCHAR   ucRel;              /* the Release level of the record      */
    UCHAR   ucMod;              /* the Modification level of the record */
  } VRM;
  #define SPM_VRM
#endif


#ifndef SPM_TYPE10_DEFINED
  /*

      Type 10: Subtype 0: Managed Session Interval Record.

        This is the basic collection record.  It is sometimes called a
        snapshot record.  It contains all of the data collected during a
        collection interval as a list of data blocks.
  */

  /*
    The interval records are constructed of the following blocks. They are
    not chained, but are recorded sequentially as they are received.
    Each interval record contains node and timing information and the
    requested collection items that were available in the interval.
    If the data for a collection interval for a Managed Session is
    large, there may be more than one record created.
  */

  /*
    the DATARECHDR is the first structure in all data collection records.

  */
  typedef struct _DCFRECHDR
  {
    BASICHDR   bhDR;           /* standard basic header */
    CHAR       szNodeName[SPM_NODENAME_SIZE]; /* Node data came from */
    FPDATETIME dtMngWrite;     /* when Monitoring session wrote the record */
    FPDATETIME dtMedStart;     /* when the Monitoring interval started on the
                                    Monitored machine */
    FPDATETIME dtMedEnd;       /* when the Monitoring interval ended on the
                                    Monitored machine */
    VRM        vrmDCF;         /* the VRM for the Service Functions at the
                                    DCF */
    SPMBOOL    fCollErrors;    /* flags indicating collection errors */
      #define  SPM_COLLERR_PERFOVFL    0x0001  /* the internal message queue
                                                  overflowed */
      #define  SPM_COLLERR_GRPSOVFL    0x0002  /* groups could not be added
                                                  to record */
      #define  SPM_COLLERR_THESEUS     0x0004  /* THESEUS2 returned an
                                                  error */
      #define  SPM_COLLERR_GROUPSMISS  0x0008  /* Not all groups were
                                                  resolved */
      #define  SPM_COLLERR_ITEMSMISS   0x0010  /* Not all items were
                                                  resolved */
      #define  SPM_COLLERR_TIMERMISS   0x0020  /* DCF cannot keep up -
                                                  interval lost */
      #define  SPM_COLLERR_SMPPTMISS   0x0040  /* a recognized sample could
                                                  not be collected */
    USHORT     usGrpCnt;       /* the number of groups in the record */
  }DATARECHDR;
  #define INTERVAL_TYPE         10
  #define INTERVAL_BASIC_STYPE  0

  /*
    the data collection record continues with structures of collected    .
    groups.
  */
  typedef struct _DCFGROUP
  {
    USHORT    usLL;            /* the length of the collected group */
    USHORT    usSPMGrp;        /* the group's ordinal */
    PID1632   pidDR;           /* pid of process if instance data Rev#002*/
    TID1632   tidDR;           /* tid of process if instance data Rev#002*/
    ROF       rofszInstance;   /* offset to text associated with the instance */
    BYTE      byDataStart;     /* start of data for this group */
    /* The actual structure for the group follows this block */
    /* It is followed by the Instance Block (if any) */
  }DCFGROUP;

  /*
    The structures for all defined SPM Groups are found in SPMCTRGP.H
    which is generated using SPMAPPIN.CMD and *.SPM files.

    A Basic Interval record for a node MIGHT look like this:
    Ŀ
     DCFRECHDR   
    Ĵ
     DCFGROUP    
    Ĵ
     cpu_data    
    Ĵ
     DCFGROUP    
    Ĵ
     memory_data 
    Ĵ
     DCFGROUP    
    Ĵ
     dasd_data   
    
    where the data types are identified by the usSPMGrp field in the
    DCFGROUP which precedes them and have the format defined in the
    SPMCTRGP.H file.
  */
  /* for ease of use, the size of a type 10 record is limited */
  #define SPM_TYPE10_MAX_SIZE (4096*4)
  #define SPM_TYPE10_DEFINED
#endif

#ifndef SPM_INSTANCEBLOCK
  /* InstanceBlock
    Each data block in a type 10 record can be either persistent or
    transient. A transient block is called an instance. It tracks event
    data such as an open file or an active thread. A ROF in the static
    section of the block points at the instance block iff the group
    represents an instance.
  */
  typedef struct _INSTANCEBLOCK
  {
    FPDATETIME      dtBirth;       /*
                                      The time that the instance was
                                      born.  This field is included in
                                      the summary criteria (sample
                                      interval < recording interval)
                                   */
    FPDATETIME      dtDeath;       /*
                                      The time that the instance died.
                                      This field is 0e0 for all
                                      intervals where the instance is
                                      still alive
                                   */
    CHAR            szzInstanceText[1]; /*
                                        One or more null terminated
                                        strings which describe the
                                        instance. The first string is
                                        always the path to the program
                                        which is involved. Additional
                                        strings indicate which resource
                                        is involved and are dependent
                                        upon the group type. The string
                                        of strings is terminated by a
                                        null string.
                                      */
  }INSTANCEBLOCK;
  #define SPM_INSTANCEBLOCK
#endif

#if defined INCL_32
  #if !defined C386
    #pragma seg16(EXTENDEDRC)
    #pragma seg16(FILTERSPEC)
    #pragma seg16(LOGFILEHANDLE)
    #pragma seg16(QUTIMES)
    #pragma seg16(QUERYHDR)
    #pragma seg16(QUILOGSTAT)
    #pragma seg16(QUCFGDTIMES)
    #pragma seg16(BASICHDR)
    #pragma seg16(VRM)
    #pragma seg16(DATARECHDR)
    #pragma seg16(DCFGROUP)
    #pragma seg16(INSTANCEBLOCK)
  #endif
  #pragma pack()
#endif

