/****************************** Module Header ******************************\
*
* Module Name: SPMUH.H
*
* OS/2 SPM 2.0 User hooks and Timer functions definitions.
*
* Copyright (c) International Business Machines Corporation 1992.
*
* ===========================================================================
*
*  This include file contains the definitions and constants for the
*  registration of a performance counter group. This header file should be
*  included when SPMRegister or any of the SPMDataItem upate functions
*  are needed.
*
*  Link with SPMUH386.LIB if you are writing a 32-bit 'flat' application and
*  with SPMUH286.LIB if you are writing a 16-bit 'segmented' application.
*  The code in SPMUH286.LIB will run on both OS/2 1.x and OS/2 2.0, although
*  it returns '0' for the time values when running on OS/2 1.x.
*
*  Sample code is included at the bottom of this file.	Scan for 'example'.
*
*
\***************************************************************************/

// Type definitions:

//***************************************************************************
//  Some toolkits do not have the following defined.
#ifndef APIENTRY
  #define APIENTRY pascal far
#endif
#ifndef VOID
  #define VOID void
  typedef VOID FAR *PVOID;
#endif
#ifndef LONG
  #define LONG long
#endif
#ifndef __IBMC__
  #ifndef INCL_32
    #ifndef INCL_16
      typedef struct _QWORD {	    // qword
	 ULONG	 ulLo;
	 ULONG	 ulHi;
      } QWORD;
      typedef QWORD FAR *PQWORD;
    #endif
  #endif
#endif
//***************************************************************************

// NOTE: Even though the QWORD is a structure with 2 ulongs, the QWORD is
//   used as an unsigned 64-bit number.

typedef ULONG	CTR;		// A simple counter (always gets bigger).
				//  Use ++ operator or SPMUpdateCtr(,+1).
typedef QWORD	TMR;		// A timer.
				//  Use SPMStartTimer() & SPMStopTimer().
typedef QWORD	BULKCTR;	// A bulk counter.
				//  Use SPMUpdateBulkCtr().
typedef ULONG	CTR_STATE;	// A "state" counter.
				//  Store into it or use SPMUpdateCtr().
typedef QWORD	TMR_STATE;	// A "state" timer.
				//  Use SPMStartTimer() & SPMStopTimer().
typedef QWORD	BULKCTR_STATE;	// A "state" bulk counter.
				//  Use SPMUpdateBulkCtr().

typedef CTR	      FAR *PCTR;
typedef TMR	      FAR *PTMR;
typedef BULKCTR       FAR *PBULKCTR;
typedef CTR_STATE     FAR *PCTR_STATE;
typedef TMR_STATE     FAR *PTMR_STATE;
typedef BULKCTR_STATE FAR *PBULKCTR_STATE;

// Queue Length (ql) definitions:
//  Use SPMEnterQ(), SPMExitQ() as a pair for normal queue functions (adding
//    elements to a queue or removing elements from a queue), or
//  SPMSetQueue() to set a new value into the queue_length.

struct	ql_s	{
	TMR	ql_tm;
	CTR	ql_ct;
		};
typedef struct	ql_s ql_t;
typedef struct	ql_s QL;
typedef QL	FAR *PQL;


#ifndef PVWXPORT
///////////////////////////////////////////////////////////////////////////////
//	SPM Block IDentifier (bid)

struct	bid_s	{
	USHORT	bid_usInstance; // Block Instance ID
	USHORT	bid_usGroup;	// Block Group ID
		};

typedef struct	bid_s	bid_t;	// Block ID
typedef struct	bid_s	BID;

typedef bid_t	FAR *pbid_t;	// Ptr to Block ID
typedef bid_t	FAR *PBID;

///////////////////////////////////////////////////////////////////////////////
//	SPM Data Block Header (dbh)

struct	dbh_s {
	ULONG	dbh_ulTotLen;	// Total Length of data block (including hdr)
	BID	dbh_bidID;	// Block IDentifier
	ULONG	dbh_flFlags;	// Flags
	ULONG	dbh_ulSem;	// Semaphore or Semaphore handle
	PVOID	dbh_reserved_1; // Reserved for use by SPM/2
	PVOID	dbh_reserved_2; // Reserved for use by SPM/2
		};

typedef struct	dbh_s	dbh_t;	// Data Block Header
typedef struct	dbh_s	DBH;

typedef dbh_t	FAR *pdbh_t;	// Pointer to Data Block Header
typedef dbh_t	FAR *PDBH;

#endif

///////////////////////////////////////////////////////////////////////////////
//	SPM Names Block

struct	spmn_s {
	PVOID  spmn_variable;		// Address of SPMDataItem
	USHORT spmn_var_size;		// Sizeof (spmn_varialbe)
	USHORT spmn_type;		// Type of spmn_variable
	PSZ    spmn_name;		// Name of spmn_variable
		   };

typedef struct spmn_s  spmn_t;		// Names Block
typedef struct spmn_s  NAMES;

typedef spmn_t FAR *pspmn_t;		   // Pointer to Names Block
typedef spmn_t FAR *PNAMES;

#define SPM_T_CTR		0
#define SPM_T_TMR		1
#define SPM_T_QL		2
#define SPM_T_BULKCTR		4
#define SPM_T_CTR_STATE 	6
#define SPM_T_TMR_STATE 	7
#define SPM_T_BULKCTR_STATE	8

#define SPMDataItem(var,typ,nam) { (PVOID) &(var), sizeof( (var)), (typ), nam }
#define SPMEnd { (PVOID) 0L, 0, 99, (PSZ) 0L }

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// Functions:

	// Registeration functions:
// To register an SPMDataGroup:
ULONG APIENTRY SPMRegister(
      PSZ    SpmSWName,        // Name of your software package.
			       //  For instance:
			       //    "IBM SPM" is our name,
			       //    "IBM CommoMgr" might be the Communication
			       //		    Manager from IBM.
      PSZ    SpmProgramName,   // Name of your .EXE file.
			       //  For instance:
			       //    "SPMT32" is the name of our 32-bit test
			       //	      program for User Hooks.
      PSZ    SpmProgramType,   // Program type:
			       //   DD,
			       //   VDD,
			       //   IFS,
			       //   WINDOWCOMPAT,
			       //   FULLSCREEN,
			       //   PM.
      PSZ    SpmComment,       // A comment.  Anything can go here.  It is used
			       //  to help identify the data.  You can use it
			       //  to carry any unique information that you
			       //  want.
      PSZ    SpmDataGroupName, // SPMDataGroup name.  It is VERY important
			       //  that this name be unique.  It is limited
			       //  to 20 characters.
      PDBH   SpmDataGroup,     // Address of SPMDataGroup block.
      PNAMES SpmNames,	       // Address of SPMNames.
      PULONG SpmGroupHandle    // Pointer to handle which is returned.	It is
			       //  passed to some of the other API's.
			  );
    // Return values are:
    //	 0 -> OK.
    //	 Low word = 1, high word = variable number that is in error.
    //	 2 -> ProgramType is not valid.
    //	 other -> ReturnCode from DosRegisterPerfCtrs (register).
// To deregister an SPMDataGroup:
ULONG APIENTRY SPMDeregister( ULONG SpmGroupHandle); // Handle for Group.
    // Return values are:
    //	 0 -> OK.
    //	 1 -> Group is not currently registered.
    //	 other -> ReturnCode from DosRegisterPerfCtrs (deregister).
// To deregister all groups registered by this process:
  // Note: This function should ONLY be used within an ExitList function.
  //	   The ExitList routine it is in must have an "order" value < 0xF0,
  //	     because SPMRegister has one registered at 0xF0 which will
  //	     deregister any that are left and will make entries in SysLog for
  //	     them.  This entry point (SPMDeregisterAll) does the same thing
  //	     without making SysLog entries.
VOID APIENTRY SPMDeregisterAll( VOID);

	// SPM Semaphore functions:
// To acquire the semaphore, before updating any DataItems:
VOID APIENTRY SPMSetSem( ULONG SpmGroupHandle); // Handle for Group.
// To release the semaphore, after  updating the DataItems:
VOID APIENTRY SPMClearSem( ULONG SpmGroupHandle); // Handle for Group.

	// SPM Counter functions:
// To update a counter:
VOID APIENTRY SPMUpdateCtr( PCTR SpmCtr,	// Address of Counter to update.
			    LONG delta);	// Delta value for counter.
// To update a bulk counter or "state" bulk counter:
VOID APIENTRY SPMUpdateBulkCtr( PBULKCTR SpmBulkCtr, // Address of Bulk Counter to update.
				LONG delta);	     // Delta value for counter.
// To add 2 QWORDs:
BOOL APIENTRY SPMAddQword( PQWORD a,   // a = a + b
			   PQWORD b);  // function return = processor carry flag
// To subtract 2 QWORDs:
BOOL APIENTRY SPMSubQword( PQWORD a,   // a = a - b
			   PQWORD b);  // function return = processor carry flag

	// SPM Timer functions:
// To start the timer.
VOID APIENTRY SPMStartTimer( PTMR pTimer);	// Address of Timer to start.
// To stop  the timer.
VOID APIENTRY SPMStopTimer( PTMR pTimer);	// Address of Timer to stop.

	// SPM QueueLength funtions:
// To add n elements to a Queue:
VOID APIENTRY SPMEnterQ( PQL   pqlQueue,	// Address of QueueLength
						//  DataItem.
			 ULONG ulDelta);	// Number of elements added to
						//  the queue.	(This is
						//  normally 1.)
// To remove n elements from a Queue:
VOID APIENTRY SPMExitQ( PQL   pqlQueue, 	// Address of QueueLength
						//  DataItem.
			ULONG ulDelta); 	// Number of elements removed
						//  from the queue.  (This is
						//  normally 1.)
// To set a value into a QueueLength DataItem:
VOID APIENTRY SPMSetQueue( PQL	   pqlQueue,	// Address of QueueLength
						//  DataItem.
			   PQWORD  pqwTime,	// Address of current time.
						// If NULL, current time will
						//  be gotten from the system.
			   LONG    ulNewCount); // New count to be placed in
						//  Queue Count.
// To get current time.
VOID APIENTRY SPMQueryTime( PQWORD pqwTime);	// Address to put current time.
// To get timer frequency.
VOID APIENTRY SPMQueryFreq( PULONG pulFreq);	// Address to put frequency.

// Macros:
	// SPM Queue DataItems:
// When adding an element to a Queue:
#define SPMAddQueueElement(Qcnt,Qlen) (Qcnt)++; SPMEnterQ( Qlen, 1);
// When removing an element from a Queue:
#define SPMRemoveQueueElement(Qcnt,Qlen) SPMExitQ( Qlen, 1);

///////////////////////////////////////////////////////////////////////////////
//		Example (actually used in our test program):
///////////////////////////////////////////////////////////////////////////////
//
//#include <os2.h>
//#include "SPMUH.h"
//
//struct _test_db {
//	  DBH		dbh;
//	  CTR		counter;
//	  TMR		timer;
//	  QL		queueLength;
//	  BULKCTR	bulkCounter;
//	  CTR_STATE	counter_state;
//	  TMR_STATE	timer_state;
//	  BULKCTR_STATE bulkcnt_state;
//		  } test_db;
//
//NAMES names[] = {
//   SPMDataItem( test_db.counter,	 SPM_T_CTR,	       "counter"),
//   SPMDataItem( test_db.timer,	 SPM_T_TMR,	       "timer"),
//   SPMDataItem( test_db.queueLength,	 SPM_T_QL,	       "queueLength"),
//   SPMDataItem( test_db.bulkCounter,	 SPM_T_BULKCTR,        "bulkCounter"),
//   SPMDataItem( test_db.counter_state, SPM_T_CTR_STATE,      "counter_state"),
//   SPMDataItem( test_db.timer_state,	 SPM_T_TMR_STATE,      "timer_state"),
//   SPMDataItem( test_db.bulkcnt_state, SPM_T_BULKCTR_STATE,  "bulkcnt_state"),
//	SPMEnd
//	};
//
//VOID main( int argc, char *argv[])
//{
//  ULONG  rc;
//  ULONG  SPMhandle;
//  QWORD  qwCurrentTime;
//
//  rc = SPMRegister( "SPM test",
//		      "SPMT32",
//		      "WindowCompat",
//		      "commEnt, and more, and still more",
//		      "testing_32",
//		      (PDBH) &test_db,
//		      (PNAMES) &names,
//		      &SPMhandle);
//  if (rc == 0)
//  {
//    // Let's update them.
//
//    // The sempahore must be set before we start updating the values.
//    SPMSetSem( SPMhandle);
//
//    // The counter can be updated.
//    SPMUpdateCtr( &test_db.counter, +1);
//    SPMUpdateCtr( &test_db.counter, -2);
//
//    // The timer is started and then stopped.
//    SPMStartTimer( &test_db.timer);
//    SPMStopTimer( &test_db.timer);
//
//    // Set the Queue_Length to zeroes before starting.
//    test_db.queueLength.ql_ct =
//    test_db.queueLength.ql_tm.ulHi =
//    test_db.queueLength.ql_tm.ulLo = 0;
//
//    // Enter and exit with various amounts.
//    SPMEnterQ( &test_db.queueLength, 1);
//    SPMEnterQ( &test_db.queueLength, 2);
//    SPMExitQ( &test_db.queueLength, 2);
//    SPMExitQ( &test_db.queueLength, 1);
//
//    // Set the Queue_Length fields back to zeroes.
//    test_db.queueLength.ql_ct =
//    test_db.queueLength.ql_tm.ulHi =
//    test_db.queueLength.ql_tm.ulLo = 0;
//
//    // Set specific values into the Queue_Length, using 'current' time.
//    SPMSetQueue( &test_db.queueLength, NULL, 4);
//    SPMSetQueue( &test_db.queueLength, NULL, 3);
//    SPMSetQueue( &test_db.queueLength, NULL, 0);
//
//    // Query current time value.
//    SPMQueryTime( &qwCurrentTime);
//
//    // Set the Queue_Length fields back to zeroes.
//    test_db.queueLength.ql_ct =
//    test_db.queueLength.ql_tm.ulHi =
//    test_db.queueLength.ql_tm.ulLo = 0;
//
//    // Set specific values into the Queue_Length, using a specified time.
//    SPMSetQueue( &test_db.queueLength, &qwCurrentTime, 4);
//    SPMSetQueue( &test_db.queueLength, &qwCurrentTime, 3);
//    SPMSetQueue( &test_db.queueLength, &qwCurrentTime, 0);
//
//    // Now that we are done, clear the semaphore.
//    SPMClearSem( SPMhandle);
//
//    // Can now deregister the group.
//    rc = SPMDeregister( SPMhandle);
//  }
//}
