;****************************** Module Header *******************************
; Module Name: SPMUH.inc
;
; OS/2 SPM 2.0 User hooks and Timer functions definitions for ring 0.
;
; Copyright (c) International Business Machines Corporation 1992
;
; ===========================================================================
;
;  This include file contains the definitions and constants for the
;  registration of a performance counter group. This header file should be
;  included when dh_RegisterPerfCtrs(), fsh_RegisterPerfCtrs() are needed.
;
;
;**************************************************************************
;  Miscellaneous constants
;  Types
CTR     struc
        dd      0       ; A 4 byte counter.
CTR     ends

TMR     struc
        dq      0       ; A timer (8 bytes).
TMR     ends

BULKCTR struc
        dq      0       ; An 8 byte counter.
BULKCTR ends

CTR_STATE struc
          dd      0     ; A 4 byte counter.
CTR_STATE ends

TMR_STATE struc
          dq      0     ; A timer (8 bytes).
TMR_STATE ends

BULKCTR_STATE struc
              dq      0 ; An 8 byte counter.
BULKCTR_STATE ends

PCTR    struc
        dd      ?       ; A pointer to a CTR.
PCTR    ends

PTMR    struc
        dd      ?       ; A pointer to a TMR.
PTMR    ends

PBULKCTR struc
         dd      ?      ; A pointer to a BULKCTR.
PBULKCTR ends

PCTR_STATE struc
           dd      ?    ; A pointer to a CTR_STATE.
PCTR_STATE ends

PTMR_STATE struc
           dd      ?    ; A pointer to a TMR_STATE.
PTMR_STATE ends

PBULKCTR_STATE struc
               dd  ?    ; A pointer to a BULKCTR_STATE.
PBULKCTR_STATE ends

;*** Queue Length (ql) definitions

ql_s    STRUC
ql_tm   dq      0       ; The timer portion of a QL.
ql_ct   DD      0       ; The queue_length portion of a QL.
ql_s    ENDS

ql_t    struc
        db      size ql_s dup(?)
ql_t    ends

QL      struc
        db      size ql_s dup(?)
QL      ends

PQL     struc
        dd      ?       ; A pointer to a QL.
PQL     ends

;*************************************************************************
;  RegisterPerfCtrs (RPC) definitions
;*
;*      For DosRegisterPerfCtrs(), dh_RegisterPerfCtrs(),
;*      fsh_RegisterPerfCtrs(), vdh_RegisterPerfCtrs().
;
;       FLAGS   RegisterPerfCtrs Flags -- valid values for 'flFlags' of Data
;*              block are:
;*
;*              {RPC_FL_16BIT | RPC_FL_32BIT} |
;*              {RPC_FL_PERS | RPC_FL_TRAN} |
;*              {RPC_FL_KRNL | RPC_FL_DD | RPC_FL_FSD | RPC_FL_VDD | RPC_FL_APP} |
;*              RPC_FL_TMR_INIT |
;*              RPC_FL_DEREG
;
RPC_FL_16BIT    EQU     0000H         ;  16-bit interface
RPC_FL_32BIT    EQU     0001H         ;  32-bit interface
RPC_FL_PERS     EQU     0002H         ;  Persistent entity type
RPC_FL_TRAN     EQU     0004H         ;  Transient entity type
RPC_FL_KRNL     EQU     0008H
RPC_FL_DD       EQU     0010H         ;  Device driver entity type
RPC_FL_FSD      EQU     0020H         ;  FSD entity type
RPC_FL_VDD      EQU     0040H         ;  VDD entity type
RPC_FL_APP      EQU     0080H         ;  Application entity type
RPC_FL_TMR_INIT EQU     0100H         ;  Init Tmr function ptrs at
;                                        * registration time.
;
RPC_FL_DEREG    EQU     0200H         ;  Deregister entity
;       The following flags are reserved for private use by the kernel; they
;*      are cleared at registration time.
;
RPC_FL_CONFIG   EQU     0400H         ;  Configured
;               Flag Masks
;
RPC_FLM_KRNL_PERS       EQU     (RPC_FL_KRNL  OR  RPC_FL_PERS)
RPC_FLM_KRNL_TRAN       EQU     (RPC_FL_KRNL  OR  RPC_FL_TRAN)
RPC_FLM_DRV     EQU     (RPC_FL_DD  OR  RPC_FL_FSD  OR  RPC_FL_VDD) ;  Driver entities
RPC_FLM_PRIVATE EQU     (RPC_FL_CONFIG) ;  Private flags for kernel use
RPC_FLM_ET      EQU     (RPC_FL_KRNL  OR  RPC_FL_DD  OR  RPC_FL_FSD  OR  RPC_FL_VDD  OR  RPC_FL_APP) ;  Entity Type


;*************************************************************************
;       Perfview Block IDentifier (bid)
;

pvbid_s STRUC
bid_usInstance  DW      ?       ;  Block Instance ID
bid_usGroup     DW      ?       ;  Block Group ID
pvbid_s ENDS

pvbid_t struc
        db      size pvbid_s dup(?)
pvbid_t ends

BID     struc
        db      size pvbid_s dup(?)
BID     ends

ppvbid_t struc
        dd      ?
ppvbid_t ends

PBID struc
        dd      ?
PBID ends

;*************************************************************************
;       PerfView Text Block Name (tbn)
;*
;*      FLAGS   Valid values for 'tbn_ulFlags' are:
;*
;*              for Counter Names:
;*
;*              {TBN_FL_CT_UNKNOWN | TBN_FL_CT_CNT | TBN_FL_CT_TIMR |
;*              TBN_FL_CT_QLEN | TBN_FL_CT_SUMSQ | TBN_FL_BULKCT}
;*
;*              for Group and Instance Names:
;*
;*              {TBN_FL_PERS | TBN_FL_TRAN} |
;*              {TBN_FL_KRNL} |
;*              {TBN_FL_MULTIPROC}
;*
;*      NOTES   The structure members take on slightly different meanings
;*              when they are used to describe a Transient Entity Instance.
;*              In this case, the tbh_atbnName member of the Text Block
;*              Header (TBH) serves as a pointer to an array of instance
;*              descriptor blocks which have the same form as pvtbName_s, but
;*              rather than describing counters in a separate Data Block,
;*              they are self-describing:  tbn_pszName describes the contents
;*              of tbn_ulFlags, tbn_usMsgID would retain its usual meaning,
;*              and tbn_usSize would be unused.
;*
;*              For example, if tbn_pszName = "PID", then tbn_ulFlags will
;*              contain the pid of the process which created this instance.
;*              Typical descriptors for kernel transient instances would be
;*              "PID", "TID", and "SLOT #".  By convention, the first TBN
;*              block of the tbh_atbnName array will contain the name of the
;*              creating process in tbn_pszName; in this case, tbn_ulFlags
;*              will be unused.
;

pvtbName_s      STRUC
tbn_ulFlags     DD      ?    ;  Flags, see TBN_FL_***
tbn_usSize      DW      ?     ;  Counter Size
tbn_usMsgID     DW      ?    ;  Message ID
tbn_pszName     DD      ?    ;  Name (default message)
pvtbName_s      ENDS

pvtbName_t struc
        db      size pvtbName_s dup(?)
pvtbName_t ends

TBN     struc
        db      size pvtbName_s dup(?)
TBN     ends

ppvtbName_t struc
        dd      ?
ppvtbName_t ends

PTBN    struc
        dd      ?
PTBN    ends

_PVWNAME EQU    pvtbName_s
PVWNAME struc
        db      size pvtbName_s dup(?)
PVWNAME ends

;       Flags values (tbn_ulFlags)
;
TBN_FLM_CT        EQU     000FH  ;  Counter Flag Mask (values 0 -> 0xF)
TBN_FL_CT_CNT     EQU     0000H  ;    Count
TBN_FL_CT_TIMR    EQU     0001H  ;    Timer
TBN_FL_CT_QLEN    EQU     0002H  ;    Queue length
TBN_FL_CT_SUMSQ   EQU     0003H  ;    Sum of Squares
TBN_FL_CT_BULKCT  EQU     0004H  ;    Bulk Counter
TBN_FL_CT_UNKNOWN EQU     0005H  ;    Unknown type
                                   ;  These are used in the tbh_tbnGroup and
;                                   * tbh_tbnInstance Name blocks (see pvtbh_s).
;
TBN_FL_PERS     EQU     0010H  ;  Persistent Entity
TBN_FL_TRAN     EQU     0020H  ;  Transient Entity
TBN_FL_KRNL     EQU     0040H  ;  Kernel Entity
TBN_FL_MULTIPROC        EQU     0080H  ;  Multiple Processor Entity
                                   ;  General purpose (per counter) flags
TBN_FL_ADVANCED EQU     0100H  ;  Advanced property
TBN_FL_HISTOGRAM        EQU     0200H  ;  Start/End histogram counters
TBN_FLM_UNKNOWN EQU     0F000H  ;  For Unknown counter types
;                                   * (TBN_FL_CT_UNKNOWN), these flags are
;                                   * reserved for use by the Registrant.
;
;       PerfView Text Block Header (tbh)
;*
;*      NOTES   Values for the version number, tbh_ulVersion, are defined
;*              below (TBH_VER_...).
;*
;*              See NOTES for Perfview Text Block Name (tbn) concerning
;*              the tbh_atbnName member.
;

pvtbh_s STRUC
tbh_ulVersion   DD      ?  ;  Version number (MUST BE FIRST MEMBER)
tbh_bidID       DB      SIZE BID DUP (?)      ;  Block IDentifier
tbh_tbnGroup    DB      SIZE TBN DUP (?)   ;  Text block Group name
tbh_tbnInstance DB      SIZE TBN DUP (?) ;  Text block Instance name
tbh_pszMsgFile  DD      ? ;  Message File name
tbh_pszHelpFile DD      ? ;  Help File name
tbh_culName     DD      ?    ;  Number of counters Names
tbh_atbnName    DD      ?   ;  Pointer to array of counter Name blocks
pvtbh_s ENDS

pvtbh_t struc
        db      size pvtbh_s dup(?)
pvtbh_t ends

TBH     struc
        db      size pvtbh_s dup(?)
TBH     ends

_PVWFSDTEXTH    EQU     pvtbh_s         ;  for File System Driver (FSD)
PVWFSDTEXTH struc
        db      size pvtbh_s dup(?)
PVWFSDTEXTH ends

ppvtbh_t struc
        dd      ?
ppvtbh_t ends

PTBH    struc
        dd      ?
PTBH    ends

NUM_TBH_PSZ     EQU     4                ;  Number of ASCIIZ ptrs in TBH
;       Version number values (tbh_ulVersion)
;*
;*      Format: A.B.C.D where   A = Major version #
;*                              B = Major release #
;*                              C = Minor release #
;*                              D = Development revision #
;
TBH_VER_2_0_0_0 EQU     02000000H               ;  2.0.0.0
;       PerfView Data Block Header (dbh)
;*
;*      FLAGS   Valid values for 'dbh_flFlags' are:
;*
;*              {RPC_FL_16BIT | RPC_FL_32BIT} |
;*              {RPC_FL_PERS | RPC_FL_TRAN} |
;*              {RPC_FL_KRNL | RPC_FL_DD | RPC_FL_FSD | RPC_FL_VDD | RPC_FL_APP} |

pvdbh_s STRUC
dbh_ulTotLen    DD      ?       ;  Total Length of data block (including hdr)
dbh_bidID       DB SIZE BID DUP (?) ;  IDentifier (Block ID)
dbh_flFlags     DD      ?       ;  Flags
dbh_ulSem       DD      ?       ;  Semaphore or Semaphore handle
dbh_pfnTmrAdd   DD      ?       ;  Ptr to timer Addition function
dbh_pfnTmrSub   DD      ?       ;  Ptr to timer Subtract function
pvdbh_s ENDS

pvdbh_t struc
        db      size pvdbh_s dup(?)
pvdbh_t ends

DBH     struc
        db      size pvdbh_s dup(?)
DBH     ends

_PVWFSDDATAH    EQU     pvdbh_s         ;  for File System Driver (FSD)
PVWFSDDATAH struc
        db      size pvdbh_s dup(?)
PVWFSDDATAH ends

ppvdbh_t struc
        dd      ?
ppvdbh_t ends

PDBH    struc
        dd      ?
PDBH    ends

;**************************************************************************
;       SPM definitions

SPM_REGISTER_DD   EQU RPC_FL_16BIT OR RPC_FL_DD OR RPC_FL_TMR_INIT
SPM_DEREGISTER_DD EQU SPM_REGISTER_DD OR RPC_FL_DEREG

SPMDataItem     MACRO   FLAG,TYPE,NAME
        DD      FLAG            ;; Type flags for PerfView.
        DW      size TYPE       ;; Size of the data item.
        DW      0               ;; Message ID (unused).
        DW      offset NAME     ;; Pointer to the name string.
        DW      seg    NAME     ;;    "
        ENDM

;**************************************************************************
;       DataItem Types
;
SPM_T_CTR          EQU  TBN_FL_CT_CNT
SPM_T_TMR          EQU  TBN_FL_CT_TIMR
SPM_T_QL           EQU  TBN_FL_CT_QLEN
SPM_T_BULKCTR      EQU  TBN_FL_CT_BULKCT
SPM_T_CTR_STATE    EQU  6
SPM_T_TMR_STATE    EQU  7
SPM_T_BULCTR_STATE EQU  8

SPMSetSem MACRO DBH
        MOV     BYTE PTR DBH.dbh_ulSem,1                              ; PTR 1680
        INC     WORD PTR DBH.dbh_ulSem+2
        ENDM

SPMClearSem MACRO DBH
        MOV     BYTE PTR DBH.dbh_ulSem,0
        ENDM

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Following are code fragments for doing the various functions:

; To declare a counter group:

;TimerStart      DD      0       ; Address of Timer Start routine
;TimerStop       DD      0       ; Address of Timer Stop  routine
;
;; PervView "group"
;DataBlock       DB      SIZE pvdbh_s dup (0)
;Counter         CTR     <>
;Timer           TMR     <>
;DataBlockEnd    EQU     $
;
;; PerfView "name block"
;str1       db   "Counter", 0
;str2       db   "Timer", 0
;
;; Group name
;GroupName  db   "SPMDD_Group", 0
;; Instance name
;InstanceName db "SPM,SPMDD,DD,0,0,This is a sample Device Driver", 0
;; Null string
;Null       db   0
;
;        even
;NameBlock  EQU  $
;           SPMDataItem SPM_T_CTR,CTR,str1
;           SPMDataItem SPM_T_TMR,TMR,str2
;
;TextBlock  dd   TBH_VER_2_0_0_0
;           dw   0               ; Block instance ID
;           dw   0               ; Block group    ID
;        ; Group name
;           dd   0               ; flags
;           dw   0               ; size
;           dw   0               ; message ID
;           dw   offset GroupName
;           dw   seg    GroupName
;        ; Instance name
;           dd   0               ; flags
;           dw   0               ; size
;           dw   0               ; message ID
;           dw   offset InstanceName
;           dw   seg    InstanceName
;        ; trailer info
;           dd   0               ; Message File
;           dd   0               ; Help File
;           dd   2               ; number of DataItems
;           dw   offset NameBlock
;           dw   seg    NameBlock

; To Register the hook group:
;    This function can only be done at Device Driver Initialization
;        MOV     EAX,OFFSET DataBlockEnd - OFFSET DataBlock
;        MOV     DataBlock.dbh_ulTotLen,EAX
;        MOV     ECX,SPM_REGISTER_DD
;        MOV     DataBlock.dbh_flFlags,ECX
;        MOV     WORD PTR DataBlock.dbh_ulSem,1   ; Set the semaphore -PTR 1680
;        MOV     WORD PTR DataBlock.dbh_ulSem+2,0 ; Clear update count-PTR 1680
;        MOV     SI,SEG    DataBlock
;        MOV     AX,OFFSET DataBlock
;        MOV     DI,SEG    TextBlock
;        MOV     BX,OFFSET TextBlock
;        MOV     DL,DevHlp_RegisterPerfCtrs
;        CALL    [DEVHLP]
;        MOV     EAX,DataBlock.dbh_pfnTmrSub
;        MOV     TimerStart,EAX
;        MOV     EAX,DataBlock.dbh_pfnTmrAdd
;        MOV     TimerStop,EAX

; To update a single counter (SPMUpdateCtr):
;        SPMSetSem DataBlock
;        INC     Counter
;        SPMClearSem DataBlock

; To update a single timer (SPMStartTimer):
;        SPMSetSem DataBlock
;        PUSH    DS
;        PUSH    OFFSET Timer
;        CALL    [TimerStart]
;        SPMClearSem DataBlock

; To update a single timer (SPMStopTimer):
;        SPMSetSem DataBlock
;        PUSH    DS
;        PUSH    OFFSET Timer
;        CALL    [TimerStop]
;        SPMClearSem DataBlock

; To "add" (SPMEnterQ) a queue element to a QL structure:
;        SPMSetSem DataBlock
;        INC     ql_ct
;        PUSH    DS
;        PUSH    OFFSET ql_tm
;        CALL    [TimerStart]
;        SPMClearSem DataBlock

; To "remove" (SPMExitQ) a queue element from a QL structure:
;        SPMSetSem DataBlock
;        DEC     ql_ct
;        PUSH    DS
;        PUSH    OFFSET ql_tm
;        CALL    [TimerStop]
;        SPMClearSem DataBlock

; To De-register the hook group:
;    This function can only be done at Device Driver Initialization
;        MOV     ECX,SPM_DEREGISTER_DD
;        MOV     DataBlock.dbh_flFlags,ECX
;        MOV     SI,SEG    DataBlock
;        MOV     AX,OFFSET DataBlock
;        XOR     DI,DI                  ; No TextBlock address
;        XOR     BX,BX
;        MOV     DL,DevHlp_RegisterPerfCtrs
;        CALL    [DEVHLP]
